@@ -6,7 +6,7 @@
 #	  /_/  |_/_/\__,_/\___/\____/_/   \__,_/\___/_/     
 #
 #
-#		ALACORDER beta 7.4.8
+#		ALACORDER beta 7.4.9.9
 #		by Sam Robson
 #
 #
@@ -17,6 +17,7 @@ import glob
 import re
 import math
 import numexpr
+import xarray
 import bottleneck
 import numpy as np
 import pandas as pd
@@ -25,18 +26,29 @@ import openpyxl
 import datetime
 import time
 import warnings
-import PyPDF2 as pypdf
+import PyPDF2
 from io import StringIO
 
 # CONFIG
 
-pd.options.display.float_format = '${:,.2f}'.format
-
-def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, save_archive=False, set_batch=0): 
+def config(in_path, out_path="", flags="", print_log=True, warn=False, save_archive=False, set_batch=0, max_input=0): 
 
+	path_input = True if isinstance(in_path,str) else False
+	input_cap = True if max_input > 0 else False
 	# Get extensions
 	out_ext: str = out_path.split(".")[-1].strip()
-	in_ext: str = in_path.split(".")[-1].strip() if len(in_path.split(".")[-1])<5 else "directory" 
+	if path_input == True:
+		in_ext: str = in_path.split(".")[-1].strip() if len(in_path.split(".")[-1])<5 else "directory" 
+	else:
+		in_ext = "object"
+		origin = "archive"
+		make = "table"
+
+	if in_ext == "txt":
+		raise Exception("Text files not supported for input! To parse text files, use alac methods.")
+
+	if out_ext == "txt":
+		print("Warning: Text files cannot be reimported to alacorder!")
 
 	if out_path == None or out_path == "":
 		out_ext = "no_export"
@@ -44,17 +56,17 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 	# Check if input path is valid
 	if in_ext != "directory" and in_ext != "pkl" and in_ext != "csv" and in_ext != "xls" and in_ext != "json" and in_ext != "xz" and in_ext != "pdf":
 		raise Exception("Input path must be to/pdf/directory/, (archive).csv, (archive).xls, (archive).json, (archive).pkl, or (archive).pkl.xz")
-	if os.path.exists(in_path) == False:
+	if os.path.exists(in_path) == False and in_ext != "object":
 		raise Exception("Input path does not exist!")
 
 	# Check if output path is valid
 	if out_ext != "pkl" and out_ext != "txt" and out_ext != "csv" and out_ext != "xls" and out_ext != "json" and out_ext != "dta" and out_ext != "xz" and out_ext != "no_export":
 		raise Exception("Output path must be .csv, .xls, .json, or .pkl.xz! (.pkl.xz only for full text archives)")
 
-	if out_ext == "txt":
-		print("WARNING: Text files cannot be reimported to alacorder!")
-
 	fromArchive = False
+	make = ""
+	origin = ""
+	contents = []
 
 	# Set read, write modes, contents
 	if in_ext == "directory" and bool(out_ext == "pkl" or out_ext == "xz" or out_ext == "txt"): 
@@ -64,30 +76,41 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 	if in_ext == "pdf":
 		in_ext == "directory"
 		contents = [in_path]
-	elif in_ext == "directory" and bool(out_ext == "xls" or out_ext == "json" or out_ext == "csv" or out_ext == "txt" or out_ext == "dta"):
+		origin = "directory"
+		make = "archive"
+	if in_ext == "directory" and bool(out_ext == "xls" or  out_ext == "no_export" or out_ext == "json" or out_ext == "csv" or out_ext == "txt" or out_ext == "dta"):
 		make = "table"
 		origin = "directory"
 		contents = glob.glob(in_path + '**/*.pdf', recursive=True)
-	elif in_ext == "pkl":
+	if in_ext == "pkl":
 		make = "table"
 		origin = "archive"
 		contents = pd.read_pickle(in_path)['AllPagesText']
 		fromArchive = True
-	elif in_ext == "xz":
+	if in_ext == "xz":
 		make = "table"
 		origin = "archive"
 		contents = pd.read_pickle(in_path,compression="xz")['AllPagesText']
 		fromArchive = True
-	elif in_ext == "csv":
+	if in_ext == "csv":
 		make = "table"
 		origin = "archive"
 		contents = pd.read_csv(in_path).tolist()
-	elif in_ext == "json": 
+	if in_ext == "json": 
 		make = "table"
 		origin = "archive"
 		contents = pd.read_json(in_path)
+	if in_ext == "object" and origin == "archive":
+		print(contents.describe(), contents.columns)
+		contents = in_path['AllPagesText']
 
-	if len(contents)==0:
+
+	if input_cap == True:
+		contents = contents[0:max_input]
+
+	case_max = len(contents)
+
+	if pd.Series(contents).shape == 0:
 		raise Exception("No cases found in input path! (" + in_path + ")")
 
 	if origin == "archive" and set_batch == 0: # set batch
@@ -96,15 +119,15 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 		batchsize = 500
 	if set_batch > 0:
 		batchsize = set_batch
-		
 
-	case_max = len(contents)
 	tot_batches = math.ceil(case_max / batchsize)
 
 	batches = np.array_split(contents, tot_batches)
 
+	batchsize = len(batches[0])
+
 	if print_log == True:
-		print(f"\nInitial configuration succeeded!\n\n{in_path} ---->\n{out_path}\n\n{case_max} cases in {tot_batches} batches")
+		print(f"\nInitial configuration succeeded!\n\n{in_path} ---->\n{out_path}\n\n{case_max} cases")
 
 	conf = pd.Series({
 		'in_path': in_path,
@@ -126,8 +149,7 @@ def config(in_path: str, out_path: str, flags="", print_log=True, warn=False, sa
 	})
 	
 	return conf
-
-# BATCH METHODS
+# conf = pd.Series({'in_path': in_path, 'out_path': out_path, 'in_ext': in_ext, 'out_ext': out_ext, 'archive': fromArchive, 'origin': origin, 'make': make, 'contents': contents, 'batches': batches, 'case_max': case_max, 'tot_batches': tot_batches, 'batchsize': batchsize, 'print_log': print_log, 'warnings': warn, 'flags': flags, 'save_archive': save_archive}) 
 
 def writeArchive(conf):
 	path_in = conf['in_path']
@@ -151,7 +173,7 @@ def writeArchive(conf):
 
 	exptime = time.time()
 	paths = pd.Series(contents)
-	allpagestext = contents
+	allpagestext = paths.map(lambda x: getPDFText(x))
 	timestamp = time.time()
 
 	c = pd.DataFrame({
@@ -182,6 +204,7 @@ def writeArchive(conf):
 		print(outputs.to_string())
 	log_complete(conf, start_time)
 	on_batch = 0
+	return outputs
 
 def writeTables(conf):
 	batches = conf['batches']
@@ -345,6 +368,7 @@ def writeTables(conf):
 		
 	log_complete(conf, start_time)
 	on_batch = 0
+	return [outputs, fees, charges]
 
 def writeFees(conf):
 	batches = conf['batches']
@@ -359,6 +383,7 @@ def writeFees(conf):
 	warn = conf['warnings']
 	contents = conf['contents']
 	batches = conf['batches']
+	from_archive = conf['archive']
 	if warn == False:
 		warnings.filterwarnings("ignore")
 	start_time = time.time()
@@ -369,7 +394,12 @@ def writeFees(conf):
 	for i, c in enumerate(batches):
 		exptime = time.time()
 		b = pd.DataFrame()
-		b['AllPagesText'] = pd.Series(c).map(lambda x: getPDFText(x))
+
+		if from_archive == True:
+			b['AllPagesText'] = contents
+		else:
+			b['AllPagesText'] = pd.Series(contents).map(lambda x: getPDFText(x))
+
 		b['CaseInfoOutputs'] = b['AllPagesText'].map(lambda x: getCaseInfo(x))
 		b['CaseNumber'] = b['CaseInfoOutputs'].map(lambda x: x[0])
 		b['FeeOutputs'] = b.index.map(lambda x: getFeeSheet(b.loc[x].AllPagesText, b.loc[x].CaseNumber))
@@ -409,13 +439,14 @@ def writeFees(conf):
 			fees.to_string(path_out)
 		elif out_ext == "dta":
 			fees.to_stata(path_out)
-		elif out_ext == "no_export" or print_log == True:
+		elif out_ext == "no_export" and print_log == True:
 			print(fees.to_string())
 		else:
 			raise Exception("Output file extension not supported! Please output to .xls, .json, or .csv")
 		on_batch += 1
 	log_complete(conf, start_time)
 	on_batch = 0
+	return fees
 
 def writeCharges(conf):
 	batches = conf['batches']
@@ -430,6 +461,8 @@ def writeCharges(conf):
 	warn = conf['warnings']
 	contents = conf['contents']
 	batches = conf['batches']
+	flag = conf['flags']
+	from_archive = conf['archive']
 	if warn == False:
 		warnings.filterwarnings("ignore")
 	start_time = time.time()
@@ -439,22 +472,37 @@ def writeCharges(conf):
 	for i, c in enumerate(batches):
 		exptime = time.time()
 		b = pd.DataFrame()
-		b['AllPagesText'] = pd.Series(c).map(lambda x: getPDFText(x))
+
+		if from_archive == True:
+			b['AllPagesText'] = contents
+		else:
+			b['AllPagesText'] = pd.Series(contents).map(lambda x: getPDFText(x))
+
 		b['CaseInfoOutputs'] = b['AllPagesText'].map(lambda x: getCaseInfo(x))
 		b['CaseNumber'] = b['CaseInfoOutputs'].map(lambda x: x[0])
 		b['ChargesOutputs'] = b.index.map(lambda x: getCharges(b.loc[x].AllPagesText, b.loc[x].CaseNumber))
 		charges['Num'] = charges['Num'].map(lambda x: pd.to_numeric(x,'ignore'))
 
+		
 		chargetabs = b['ChargesOutputs'].map(lambda x: x[17])
 		chargetabs = chargetabs.dropna()
+
+
 		charges = charges.dropna()
 		chargetabs = chargetabs.tolist()
 		chargetabs = pd.concat(chargetabs,axis=0,ignore_index=True)
 		charges = charges.append(chargetabs,ignore_index=True)
-
-		
 		charges.fillna('',inplace=True)
 
+		if flag == "filing":
+			is_disp = charges['Disposition']
+			is_filing = is_disp.map(lambda x: False if x == True else True)
+			charges = charges[is_disp]
+
+		if flag == "disposition":
+			is_disp = charges['Disposition']
+			charges = charges[is_disp]
+
 		# write 
 		if out_ext == "xls":
 			with pd.ExcelWriter(path_out) as writer:
@@ -478,8 +526,10 @@ def writeCharges(conf):
 		else:
 			raise Exception("Output file extension not supported! Please output to .xls, .json, or .csv")
 		on_batch += 1
+
 	log_complete(conf, start_time)
 	on_batch = 0
+	return charges
 
 def search(conf, method, status=''): 
 	batches = conf['batches']
@@ -494,6 +544,7 @@ def search(conf, method, status=''):
 	warn = conf['warnings']
 	contents = conf['contents']
 	batches = conf['batches']
+	from_archive = conf['archive']
 	if warn == False:
 		warnings.filterwarnings("ignore")
 	start_time = time.time()
@@ -503,7 +554,12 @@ def search(conf, method, status=''):
 	for i, c in enumerate(batches):
 		exptime = time.time()
 		b = pd.DataFrame()
-		allpagestext = pd.Series(c).map(lambda x: getPDFText(x))
+
+		if from_archive == True:
+			allpagestext = contents
+		else:
+			allpagestext = pd.Series(contents).map(lambda x: getPDFText(x))
+
 		customoutputs = allpagestext.map(lambda x: method(x))
 		allCustomOutputs = allCustomOutputs.append(customoutputs)
 		if print_log == True:
@@ -525,6 +581,7 @@ def write(conf, method, status=''):
 	warn = conf['warnings']
 	contents = conf['contents']
 	batches = conf['batches']
+	from_archive = conf['archive']
 	if warn == False:
 		warnings.filterwarnings("ignore")
 	start_time = time.time()
@@ -534,7 +591,12 @@ def write(conf, method, status=''):
 	for i, c in enumerate(batches):
 		exptime = time.time()
 		b = pd.DataFrame()
-		allpagestext = pd.Series(c).map(lambda x: getPDFText(x))
+
+		if from_archive == True:
+			allpagestext = contents
+		else:
+			allpagestext = pd.Series(contents).map(lambda x: getPDFText(x))
+
 		customoutputs = allpagestext.map(lambda x: method(x))
 		alloutputs = alloutputs.append(customoutputs)
 		if print_log == True and out_ext != "no_export":
@@ -565,12 +627,13 @@ def write(conf, method, status=''):
 		on_batch += 1
 	log_complete(conf, start_time)
 	on_batch = 0
+	return alloutputs
 
 # CASE METHODS 
 
 def getPDFText(path: str) -> str:
 	text = ""
-	pdf = pypdf.PdfReader(path)
+	pdf = PyPDF2.PdfReader(path)
 	for pg in pdf.pages:
 		text += pg.extract_text()
 	return text
@@ -646,6 +709,7 @@ def getCaseInfo(text: str):
 	try:
 		dob: str = re.search(r'(\d{2}/\d{2}/\d{4})(?:.{0,5}DOB\:)', str(text), re.DOTALL).group(1)
 		phone: str = re.search(r'(?:Phone\:)(.*?)(?:Country)', str(text), re.DOTALL).group(1).strip()
+		phone = re.sub(r'[^0-9]','',phone)
 		if len(phone) < 7:
 			phone = ""
 		if len(phone) > 10 and phone[-3:] == "000":
@@ -660,7 +724,7 @@ def getCaseInfo(text: str):
 	except (IndexError, AttributeError):
 		pass
 	try:
-		street_addr = re.search(r'(Address 1\:)(.+)', str(text), re.MULTILINE).group(2).strip()
+		street_addr = re.search(r'(Address 1\:)(.+)(?:Phone)*?', str(text), re.MULTILINE).group(2).strip()
 	except (IndexError, AttributeError):
 		street_addr = ""
 	try:
@@ -677,6 +741,10 @@ def getCaseInfo(text: str):
 		state = ""
 	
 	address = street_addr + " " + city + ", " + state + " " + zip_code
+	if len(address) < 5:
+		address = ""
+	address = address.replace("00000-0000","").replace("%","").strip()
+	address = re.sub(r'([A-Z]{1}[a-z]+)','',address)
 	case = [case_num, name, alias, dob, race, sex, address, phone]
 	return case
 
@@ -707,6 +775,7 @@ def getFeeSheet(text: str, cnum: str):
 		drows = fees_noalpha.map(lambda x: x.replace(",","").split("$"))
 		coderows = srows.map(lambda x: str(x[5]).strip() if len(x)>5 else "")
 		payorrows = srows.map(lambda x: str(x[6]).strip() if len(x)>6 else "")
+		payorrows = payorrows.map(lambda x: re.sub(r'(\$\d{1,2}?\.{1}\d{2}+)','',x))
 		amtduerows = drows.map(lambda x: str(x[1]).strip() if len(x)>1 else "")
 		amtpaidrows = drows.map(lambda x: str(x[2]).strip() if len(x)>2 else "")
 		balancerows = drows.map(lambda x: str(x[-1]).strip() if len(x)>5 else "")
@@ -754,8 +823,21 @@ def getFeeSheet(text: str, cnum: str):
 		return [tdue, tbal, d999, owe_codes, codes, allrowstr, feesheet]
 
 def getCharges(text: str, cnum: str):
-	# get all charges matches
-	c = re.findall(r'(\d{3}\s{1}.{1,100}?.{3}-.{3}-.{3}.{10,75})', text, re.MULTILINE)
+
+	rc = re.findall(r'(\d{3}\s{1}.{1,100}?.{3}-.{3}-.{3}.{10,75})', text, re.MULTILINE)
+	unclean = pd.DataFrame({'Raw':rc})
+	unclean['FailTimeTest'] = unclean['Raw'].map(lambda x: bool(re.search(r'([0-9]{1}\:[0-9]{2})', x)))
+	unclean['FailNumTest'] = unclean['Raw'].map(lambda x: False if bool(re.search(r'([0-9]{3}\s{1}.{4}\s{1})',x)) else True)
+	unclean['Fail'] = unclean.index.map(lambda x: unclean['FailTimeTest'][x] == True or unclean['FailNumTest'][x]== True)
+	passed = pd.Series(unclean[unclean['Fail']==False]['Raw'].dropna().explode().tolist())
+	passed = passed.explode()
+	passed = passed.dropna()
+	passed = pd.Series(passed.tolist())
+	passed = passed.map(lambda x: re.sub(r'(\s+[0-1]{1}$)', '',x))
+	passed = passed.map(lambda x: re.sub(r'([©|\w]{1}[a-z]+)', ' ',x))
+	passed = passed.explode()
+	c = passed.dropna().tolist()
+
 	# print(c)
 	cind = range(0, len(c))
 	charges = pd.DataFrame({ 'Charges': c,'parentheses':'','decimals':''},index=cind)
@@ -800,15 +882,31 @@ def getCharges(text: str, cnum: str):
 
 	charges['TypeDescription'] = charges['Charges'].map(lambda x: re.search(r'(BOND|FELONY|MISDEMEANOR|OTHER|TRAFFIC|VIOLATION)', x).group() if bool(re.search(r'(BOND|FELONY|MISDEMEANOR|OTHER|TRAFFIC|VIOLATION)', x)) else "")
 	charges['Category'] = charges['Charges'].map(lambda x: re.search(r'(ALCOHOL|BOND|CONSERVATION|DOCKET|DRUG|GOVERNMENT|HEALTH|MUNICIPAL|OTHER|PERSONAL|PROPERTY|SEX|TRAFFIC)', x).group() if bool(re.search(r'(ALCOHOL|BOND|CONSERVATION|DOCKET|DRUG|GOVERNMENT|HEALTH|MUNICIPAL|OTHER|PERSONAL|PROPERTY|SEX|TRAFFIC)', x)) else "")
+
 	try:
 		charges['Description'] = charges['Charges'].map(lambda x: re.search(r'(\d{3}\s[\w\d]{4}\s)(.{5,75}?)(.{3}-.{3}-.{3})',x, re.MULTILINE).group(2).strip() if bool(re.search(r'(\d{3}\s[\w\d]{4}\s)(.{5,75}?)(.{3}-.{3}-.{3})',x, re.MULTILINE).group(2).replace("\n"," ").strip()) else x)
 	except (AttributeError, IndexError):
 		charges['Description'] = charges['Charges']
+
 	charges['Charges'] = charges['Charges'].map(lambda x: x.replace("SentencesSentence","").replace("Sentence","").strip())
 	charges.drop(columns=['PardonCode','PermanentCode','CERVCode','VRRexception','parentheses','decimals'], inplace=True)
-	# print(charges['Description'])
+
 	charges['Description'] = charges['Description'].map(lambda x: x.replace("\'","").strip())
 	charges['Description'] = charges.index.map(lambda x: charges['Description'][x].replace(charges['CourtActionDate'][x],"").replace(charges['CourtAction'][x],"").strip())
+
+	###
+	### TEST:
+	charges['Description'] = charges['Description'].astype(str)
+	charges['Desc2'] = charges['Description'].str.replace("TRAFFIC","").str.replace("PROPERTY","").str.replace("MISDEMEANOR","").str.replace("FELONY","").str.replace("DRUG","").str.replace("PERSONAL","")
+	charges['DescFix'] = charges.index.map(lambda x: len(charges['Description'][x]) - len(charges['Desc2'][x]))
+	charges['DescFix'] = charges.index.map(lambda x: True if charges['Description'][x].strip() == "FELONY PROPERTY" else charges['DescFix'][x])
+	charges['Description'] = charges.index.map(lambda x: re.split(r'.{3}-.{3}-.{3}',charges.Charges[x])[0] if charges.DescFix[x] > 0 else charges['Description'][x])
+	charges['Description'] = charges.index.map(lambda x: re.sub(r'(\d{3}\s[\w\d]{4})','',x) if bool(re.search(r'(\d{3}\s[\w\d]{4})','',x)) else x)
+	charges.drop(columns=['Desc2','DescFix'],inplace=True)
+
+	###
+	###
+
 	charges['Category'] = charges['Category'].astype("category")
 	charges['TypeDescription'] = charges['TypeDescription'].astype("category")
 	charges['Code'] = charges['Code'].astype("category")
@@ -859,7 +957,7 @@ def log_complete(conf, start_time):
 /_/  |_/_/\\__,_/\\___/\\____/_/   \\__,_/\\___/_/     
 																																										
 	
-	ALACORDER beta 7.4.8
+	ALACORDER beta 7.4.9.9
 	by Sam Robson	
 
 	Searched {path_in} 
